# -*- coding: utf-8 -*-
"""
Functions to generate a graph representing citations between multiple ACS/Nature journals

"""

__authors__ = "Donna Löding, Alina Molkentin, Xinyi Tang, Judith Große, Malte Schokolowski"
__email__ = "cis-project2021@zbh.uni-hamburg.de"
__status__ = "Production"
#__copyright__ = ""
#__credits__ = ["", "", "", ""]
#__license__ = ""
#__version__ = ""
#__maintainer__ = ""

from bs4 import BeautifulSoup as bs
import requests as req
import sys  
from pathlib import Path
from input_fj import input
from input_test import input_test_func
from json_demo import output_to_json

# adds every publication from input list to graph structure
# doi_input_list: list of publication dois from user

# TO-DO: Listenelemente auf Korrektheit überprüfen
def initialize_nodes_list(doi_input_list):
    for pub_doi in doi_input_list:
        pub = input_test_func(pub_doi)
        not_in_nodes = True
        for node in nodes:                                              # checks if a pub is already in nodes
            if (pub.doi_url == node.doi_url):
                not_in_nodes = False
                break
        if (not_in_nodes):
            nodes.append(pub)
            pub.group = "input"
        else:
            doi_input_list.remove(pub_doi)



# adds a node for every publication unknown
# adds edges for citations between publications     
def create_graph_structure_citations(pub, search_height, search_height_max):
    for citation in pub.citations:
        not_in_nodes = True
        for node in nodes:
            # checks every citation for duplication 
            if (citation.doi_url == node.doi_url):
                not_in_nodes = False
                break
        if (not_in_nodes):
            if (search_height <= search_height_max):
                citation.group = "height"
                nodes.append(citation)
                edges.append([pub.doi_url,citation.doi_url])

        # adds only edge if citation already exists         
        else:
            edges.append([pub.doi_url,citation.doi_url])    



# adds a node for every publication unknown
# adds edges for references between publications     
def create_graph_structure_references(pub, search_depth, search_depth_max):
    for reference in pub.references:
        not_in_nodes = True
        for node in nodes:
            # checks every reference for duplication 
            if (reference.doi_url == node.doi_url):
                not_in_nodes = False
                break
        if (not_in_nodes):
            if (search_depth <= search_depth_max):
                reference.group = "depth"
                nodes.append(reference)
                edges.append([reference.doi_url,pub.doi_url])

        # adds only edge if citation already exists           
        else:
            edges.append([reference.doi_url,pub.doi_url])       
    


# recursive function to implement height-first-search on citations
# doi_citations: input list of citet dois
# search_height: current search_height of height-first-search
# search_height_max: maximal search_height for dfs
def process_citations_rec(doi_citations, search_height, search_height_max):  
    # height of search is increased by 1 with each recursive call
    search_height += 1

    # create class object for every citation from list
    for pub_doi in doi_citations:
        pub = input_test_func(pub_doi)
        create_graph_structure_citations(pub, search_height, search_height_max)    
        # If the maximum height has not yet been reached, all references from the publication 
        # are written to an array and the function is called again with this array.       
        if (search_height < search_height_max):
            citations_list = []
            for citation in pub.citations:

                # currently only the references with acs are stored in the URL, because we can't 
                # extract the info from other sources.
                if ("acs" in citation.doi_url):
                    citations_list.append(citation.doi_url)

            # recursive call of function.
            process_citations_rec(citations_list, search_height, search_height_max)



# recursive function to implement height-first-search on references
# doi_references: input list of referenced dois
# search_depth: current search_depth of height-first-search
# search_depth_max: maximal search_depth for dfs
def process_references_rec(doi_references, search_depth, search_depth_max):  
    # The depth is increased by 1 with each recursive call
    search_depth += 1

    # create class object for every citation from list
    for pub_doi in doi_references:
        pub = input_test_func(pub_doi)
        create_graph_structure_references(pub, search_depth, search_depth_max)    
        # If the maximum depth has not yet been reached, all references from the publication 
        # are written to an array and the function is called again with this array.       
        if (search_depth < search_depth_max):
            references_list = []
            for reference in pub.references:

                # currently only the references with acs are stored in the URL, because we can't 
                # extract the info from other sources. 
                if ("acs" in reference.doi_url):
                    references_list.append(reference.doi_url)

            # recursive call of function.
            process_references_rec(references_list, search_depth, search_depth_max)



           
def process_main(doi_input_list, search_height, search_depth):
    # ERROR-Handling doi_array = NULL
    if (len(doi_input_list) == 0):
        print("Error, no input data")

    # ERROR- if a negative number is entered for height
    if (search_height < 0):
        print("Error, search_height of search must be positive")

    # ERROR- if a negative number is entered for depth
    if (search_depth < 0):
        print("Error, search_depth of search must be positive")       

    # create empty array for the nodes
    # create empty array for the edges
    global nodes, edges
    nodes = []
    edges = []

    initialize_nodes_list(doi_input_list)
    process_citations_rec(doi_input_list, 0, search_height)
    process_references_rec(doi_input_list, 0, search_depth)

    output_to_json(nodes,edges)

    # only for internal testing
    return(nodes,edges)


    
# program test, because there is no connection to the input yet.
def test_print():
    arr = []
    #arr.append('https://pubs.acs.org/doi/10.1021/acs.jcim.9b00249')
    #arr.append('https://pubs.acs.org/doi/10.1021/acs.jcim.9b00249')
    #arr.append('https://doi.org/10.1021/acs.jmedchem.0c01332')
    #arr.append('https://doi.org/10.1021/acs.jcim.0c00741')

    #arr.append('https://doi.org/10.1021/ci700007b')
    #arr.append('https://doi.org/10.1021/acs.jcim.5b00292')
    #url = sys.argv[1]
    #arr.append[url]

    arr.append('doi1')
    #arr.append('doi2')
    #arr.append('doi3')


    nodes,edges = process_main(arr,1,1)

    print("Knoten:\n")
    for node in nodes:
        print(node.title, "\n")
    print("\n Kanten:\n")
    for edge in edges:
        print(edge,"\n")

test_print()
        